import datetime

from mongoengine import *

from .material import Material


class CyclicIndirectTensileTest(Document):

    date = DateTimeField(default=datetime.datetime.now,
                         wtf_options={"render_kw": {
                             "step": "60"
                         }})

    standard = StringField(default='TP Asphalt Teil 24')

    lab = StringField(default='TU Dresden', required=True)
    auditor = StringField(default=None)
    machine = StringField(default=None)

    filehash = StringField(required=True)

    project = StringField(required=True)
    workpackage = StringField()
    material = ReferenceField(Material, required=True)

    meta = {
        'allow_inheritance': True,
        'index_opts': {},
        'index_background': True,
        'index_cls': False,
        'auto_create_index': True,
        'collection': 'citt',
    }


class CITTSiffness(CyclicIndirectTensileTest):

    #metadata
    f_set = FloatField()
    sigma_set = FloatField()
    T_set = FloatField()

    #results
    stiffness = FloatField()
    #fit parameter
    ## F
    fit_a_F = FloatField()
    fit_b_F = FloatField()
    fit_d_F = FloatField()
    fit_e_F = FloatField()
    fit_f_F = FloatField()
    r2_F = FloatField()
    ## S1
    fit_a_s_hor_1 = FloatField()
    fit_b_s_hor_1 = FloatField()
    fit_d_s_hor_1 = FloatField()
    fit_e_s_hor_1 = FloatField()
    fit_f_s_hor_1 = FloatField()
    r2_s_hor_1 = FloatField()
    ## S2
    fit_a_s_hor_2 = FloatField()
    fit_b_s_hor_2 = FloatField()
    fit_d_s_hor_2 = FloatField()
    fit_e_s_hor_2 = FloatField()
    fit_f_s_hor_2 = FloatField()
    r2_s_hor_2 = FloatField()
    ## S-Sum
    fit_a_s_hor_sum = FloatField()
    fit_b_s_hor_sum = FloatField()
    fit_d_s_hor_sum = FloatField()
    fit_e_s_hor_sum = FloatField()
    fit_f_s_hor_sum = FloatField()
    r2_s_hor_sum = FloatField()
    # data
    time = ListField(FloatField())
    F = ListField(FloatField())
    N = ListField(IntField())
    s_hor_1 = ListField(FloatField())
    s_hor_2 = ListField(FloatField())
